# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8-80 compliant>

"""
This script exports a Mesh to a RoR truck data file.

the truck format is specialized for the vehicles from the game RigsOfRods

Usage:
Execute this script from the "File->Export" menu. You can select

"""
import os
import bpy

projectname = bpy.path.basename(os.path.splitext(bpy.context.blend_data.filepath)[0])
facetype = ""
reoNodeslist = []
count = 0

def incCount():
	global count    # Needed to modify global copy of globvar
	count = count+1

def getHeader(mesh):
	text = ""
	if projectname != "" :
		text = projectname + "\n\n"
	else :
		text = "truckname\n\n"
	text = text+"guid\n\n"
	text = text+"fileformatversion 3\n\n"
	text = text+";author <Art> <AuthorID> <Authorname> <E-Mail>\n"
	text = text+"authors chassis ID YourName eMail\n"
	text = text+"authors texture ID YourName eMail\n"
	text = text+"authors support ID YourName eMail\n\n"
	text = text+"description\n"
	text = text+";the first 3 rows will be displayed in truck hud, to display press STRG+T in game\n"
	text = text+"\n"
	text = text+"end_description\n\n"
	text = text+"globals\n"
	text = text+";dry mass, cargo mass, material\n"
	text = text+"100.0,  100.0,     tracks/beamblack\n\n"
	text = text+"engine\n"
	text = text+";min rpm, max rpm, torque, differential ratio, rear gear, first, second, third, fourth, fifth, sixth ...\n"
	text = text+"1200.000000, 9000.000000, 320.000000, 3.000000, 3.5, 5.00000, 4.40000, 3.40000, 2.80000, 2.10000, 1.500000, -1.000000 \n"
	return text

def getFooter():
	text="end\n"
	return text

def sortNodes(mesh):
	text = ""

	for verts in mesh.vertices:
		text=str(count)
		incCount()
		reoNodeslist.append(text)

def getEdges(mesh):
	text="beams\n"
	for edge in mesh.edges:
		text =(text+reoNodeslist[edge.vertices[0]]+","+reoNodeslist[edge.vertices[1]]+"\n")
	text = text + "\n" + "cameras\n"
	text = text + ";center, back, left\n"
	text = text + "center-node , centerback-node , centerleft-node\n"
	return text

def getCineCam():
	text = "cinecam\n"
	text = text + ";  x,   y,   z, <---------8 bindings-------->, spring, damp\n"
	return text

def getVertices(mesh):
	text="nodes\n"

	for verts in mesh.vertices:
		text = (text+reoNodeslist[verts.index]+", "+str(round(verts.co[0],4))+", "+str(round(verts.co[2],4))+", "+str(round(verts.co[1],4))+"\n")

	return text

def triangulateFaces(mesh, poly):

	try:
		text = reoNodeslist[poly.vertices[0]]+","+reoNodeslist[poly.vertices[2]]+","+reoNodeslist[poly.vertices[1]]+","+facetype+"\n"    
		text = text + reoNodeslist[poly.vertices[2]] + "," + reoNodeslist[poly.vertices[0]]+","+reoNodeslist[poly.vertices[3]]+","+facetype+"\n"
		return text
	except:
		text = reoNodeslist[poly.vertices[0]]+","+reoNodeslist[poly.vertices[2]]+","+reoNodeslist[poly.vertices[1]]+","+facetype+"\n"     
		return text

def getFaces(mesh):
	text = ""
	text = text + "submesh\n"
	text = text + "cab\n"
	for triangle in mesh.polygons: 
		text = text+triangulateFaces(mesh, triangle)
	text = text + "backmesh\n"
	text = text + "; backmesh is set just for testing, should be removed when not needed"
	return text

def getCameraPosition(scene):
	text = ""
	return text

def buildtruckData(shape):
	sortNodes(shape)
	text = ""
	text = text + getHeader(shape)+"\n"
	text = text + getVertices(shape)+"\n"
	text = text + getEdges(shape)+"\n"
	text = text + getCineCam()+"\n"
	text = text + getFaces(shape)+"\n"
	text = text + getFooter()
	return text

def findMaterials():
	text = ""
	for mat in bpy.data.materials:
		for tex in bpy.data.textures:
			if tex.type == 'IMAGE':
				if tex.name == mat.active_texture.name:
					if tex.image is not None:
						text = text+"singleton Material(" + mat.name +")\n{\n"
						text = text+"   mapTo = \"" + mat.name +"\";\n"
						text = text+"   diffuseMap[0] = \"/vehicles/"+projectname+"/"+tex.image.name+"\"\n"
						text = text+"   specular[0] = \"0.5 0.5 0.5 1\";\n"
						text = text+"   specularPower[0] = \"50\";\n"
						text = text+"   translucentBlendOp = \"None\";\n"
						text = text+"\n}\n\n"
	return text

def write_MatFile(context, filepath):
	matFile = "";
	matFile = filepath.replace('.truck', '.material')
	f = open(matFile, 'w', encoding='utf-8')
	f.write(findMaterials())

	f.close()


def write_truck_data(context, filepath):
	
	for shape in bpy.data.meshes:
		f = open(filepath, 'w', encoding='utf-8')
		f.write(buildtruckData(shape))
		f.close()

	return {'FINISHED'}


